#include "descriptor/descriptor.h"
#define ARRAYSIZE(array) (sizeof(array)/sizeof(*(array)))
#define FRAMES 100
#define PA 0.5f
#define PD 0.7f
#define DEGTORAD 0.0174532925f
 
ITEM	asfalto,acqua, yellow,blu, legno, bianco, nero, willy1, willy2, acciaio, rock, temp, verde;	//Attributes
ITEM	wily, terra, strada, mondo, cata, masso, cactus, pozza; 					//Primitives
ITEM	sole;											//Lights
ITEM	scene;												//Scene

struct
{
   ITEM * attr;
   char * name; 
   short red;
   short green;
   short blue;
   float pa;
   float pd;
} attributes[]={
	{&asfalto, "asfalto", 89, 94, 99, 0.4, 0.4},
	{&yellow, "yellow", 255, 255, 48, 0.6, 0.6},
	{&acqua, "acqua", 173, 219, 255,0.5,0.5},
	{&bianco, "bianco", 255, 255, 255, PA, PD},
	{&nero, "nero", 0, 0, 0, 0.5, 0.7},
	{&willy1, "willy1", 128, 77, 12, PA, PD},
	{&willy2, "willy2",250, 178 , 60, PA, PD},
	{&blu, "blu", 0, 0, 255, 0.5, 0.5},
	{&legno, "legno", 130,50,0,0.5,0.5},
	{&acciaio, "acciaio", 170,173,178,0.8,0.8},
	{&verde, "verde", 60,150,0,0.8,0.8},
	{&rock, "roccia", 150, 12, 12, 0.5, 0.5} //{&rock, "roccia", 160, 90, 65, 0.5, 0.5}
};

struct {
	ITEM * attr;
	float ps; //specular reflected light
	float n;  //Phong's exponent for highlighting from direct light sources (the greater the exponent, the narrower the highlight)
} 
reflex[] = {
	{&acciaio,0.24,5},
	{&acqua, 0.3, 5}
};

struct 
{
	ITEM *attr;	// formula: t= (maxt
	float maxt;	// massima trasparenza
	float mint;	// minima trasparenza
	int tpwr;	// fattore a potenza della trasparenza
	float ri;	// indice di riflessione
} 
transparency[]={
	{&acqua, 0.8, 0.6, 1, 1.33}
};

struct primitives
{
	char * path;
	ITEM * attr; //attributo imposto alla superfice
	char * texture;
	short type; //0 con set_projection, 1 con set_domain, 2 set_bump (se texture != "")
	int r_flag;
	float pa;
	float pd;

};

struct primitives catapulta[] = {
	{"cata1A.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata1B.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata1C.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata2A.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata2B.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata2C.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata3A.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata3B.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata3C.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata4A.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata4B.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata4C.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata5A.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata5B.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata5C.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata6A.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata6B.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"cata6C.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"corda.db", &nero, ""},
	{"ruota1.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"ruota2.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"ruota3.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"ruota4.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"timone.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"rulloCorda.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"timoneA.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"timoneB.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"timoneC.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"timoneD.db", &legno,"textures/wood.hr", 1, TRUE, PA, PD},
	{"timoneE.db", &rock, "textures/wood.hr", 1, TRUE, PA, PD}
};

struct primitives willly[] = {
	{"naso2.db", &nero,""},
	{"naso.db", &willy2,""},
	{"guanciaDx.db", &willy2,""},
	{"guanciaSx.db", &willy2,""},
	{"occhioSx.db", &bianco,""},
	{"occhioDx.db", &bianco,""},
	{"pupillaSx.db", &nero,""},
	{"pupillaDx.db", &nero,""},
	{"testa.db", &willy1,""},
	{"orecchioNewSx.db", &willy1,"textures/orecchieNew.hr",  1, FALSE, PA, PD },
	{"orecchioNewDx.db", &willy1,"textures/orecchieNew.hr",  1, FALSE, PA, PD },
	{"cigliaSx.db", &willy2,""},
	{"cigliaDx.db", &willy2,""},
	{"ditoPiedeDx1.db", &willy1,""},
	{"ditoPiedeDx2.db", &willy1,""},
	{"ditoPiedeDx3.db", &willy1,""},
	{"ditoPiedeSx1.db", &willy1,""},
	{"ditoPiedeSx2.db", &willy1,""},
	{"ditoPiedeSx3.db", &willy1,""},
	{"piantaPiedeDxA.db", &willy1,""},
	{"piantaPiedeDxB.db", &willy1,""},
	{"piantaPiedeSxA.db", &willy1,""},
	{"piantaPiedeSxB.db", &willy1,""},
	{"piantaManoDx.db", &willy1,""},
	{"piantaManoSx.db", &willy1,""},
	{"gambaSx.db", &willy1,""},
	{"gambaDx.db", &willy1,""},
	{"collo.db", &willy1,""},
	{"coda.db", &willy1,""},
	{"braccioSx.db", &willy1,""},
	{"braccioDx.db", &willy1,""},
	{"busto.db", &willy1,"textures/busto.hr", 1, FALSE, PA, PD},
	{"ditoManoDx1.db", &willy1,""},
	{"ditoManoDx2.db", &willy1,""},
	{"ditoManoDx3.db", &willy1,""},
	{"ditoManoSx1.db", &willy1,""},
	{"ditoManoSx2.db", &willy1,""},
	{"ditoManoSx3.db", &willy1,""}
};

int main(void) {
	int i, j, k,l, segno;
	FILE *file;
	set_background(0.676,0.861,0.953,TRUE);
	set_ambient_light(1,1,1,0.7);
 	sole=create_point_light(55, -55,55,1,1,1,4,90, "sole");

	for (i = 0; i < ARRAYSIZE(attributes); i++){
		*(attributes[i].attr) =  create_attribute(attributes[i].name);
		set_color (	*attributes[i].attr, 
				attributes[i].red/255.000, 
				attributes[i].green/255.000, 
				attributes[i].blue/255.000, 
				attributes[i].pa, 
				attributes[i].pd);
	}

	for (i = 0; i < ARRAYSIZE(reflex); i++){
		set_reflectivity(*reflex[i].attr,reflex[i].ps, reflex[i].n);
	}
	
	for (i = 0; i < ARRAYSIZE(transparency); i++){
		set_transparency(*transparency[i].attr, transparency[i].maxt,transparency[i].mint, transparency[i].tpwr, transparency[i].ri);
	}
	
	for (i = 0; i < ARRAYSIZE(willly); i++) {
		temp = read_nurbs(willly[i].path);
		set_attribute(temp, *willly[i].attr);

		if (strcmp(willly[i].texture,"")) // se texture è diverso da "" (stringa vuota)
			switch (willly[i].type) {
			case 0:
				set_projection_texture(temp,*willly[i].attr,willly[i].texture,0,willly[i].r_flag,willly[i].pa,willly[i].pd);
				break;
			case 1:
				set_domain_texture(temp,*willly[i].attr,willly[i].texture,0,1,0,1,willly[i].r_flag,willly[i].pa,willly[i].pd);
				break;
			case 2:
				set_bump_texture(temp,*willly[i].attr,willly[i].texture,0,1,0,1,willly[i].r_flag,willly[i].pa,willly[i].pd);
				break;
			}

// 		set_z_rotate(temp, 180, TRUE);
		set_translate(temp, 17,5,0.36);

		if (i == 0) { 
			wily = create_list(temp, "wily");
		} 
		else {
			wily = insert_in_list(wily, temp); 
		}
	}

	for (i = 0; i < ARRAYSIZE(catapulta); i++) {
		temp = read_nurbs(catapulta[i].path);
		set_attribute(temp, *catapulta[i].attr);

		if (strcmp(catapulta[i].texture,"")) 
			switch ((catapulta[i].type)) {
			case 0:
				set_projection_texture(temp,*catapulta[i].attr,catapulta[i].texture,0,catapulta[i].r_flag,catapulta[i].pa,catapulta[i].pd);
				break;
			case 1:
				set_domain_texture(temp,*catapulta[i].attr,catapulta[i].texture,0,1,0,1,catapulta[i].r_flag,catapulta[i].pa,catapulta[i].pd);
				break;
			case 2:
				set_bump_texture(temp,*catapulta[i].attr,catapulta[i].texture,0,1,0,1,catapulta[i].r_flag,catapulta[i].pa,catapulta[i].pd);
				break;
			}

		set_z_rotate(temp, 90, TRUE);
		set_translate(temp, 0.,5.,1.);

		if (i == 0) {  
			cata = create_list(temp, "catapulta");	
		}
		else {
			cata = insert_in_list(cata, temp); 
		}
	}

	terra = read_nurbs("terra.db");
	mondo = read_nurbs("mondo.db");
	strada = read_nurbs("strada.db");
	pozza = read_nurbs("pozza2.db");
	set_attribute(pozza, acqua);
	temp = read_nurbs("cactusA.db");
	set_domain_texture(temp, verde,"textures/cactus.hr",0,1,0,1,TRUE,0.5,0.7);
	cactus = create_list(temp, "cactus");
	temp = read_nurbs("cactusB.db");
	set_domain_texture(temp, verde,"textures/cactus2.hr",0,1,0,1,TRUE,0.5,0.7);
	cactus = insert_in_list(cactus, temp);
	set_scale(mondo, 2, 2, 2);
	set_scale(terra, 2, 2, 1);
	set_scale(strada, 2,2,1);
	set_scale(cactus, 2, 2, 2);
	set_scale(pozza, 2,2,1);
	set_translate(cactus, -15, 0, 0);
	set_attribute(strada, asfalto);
	set_domain_texture(strada, asfalto, "textures/asfalto2.hr",0,.1,0,.1,TRUE,0.2,0.4);
	set_domain_texture(terra,yellow,"textures/sand2a.hr",0,1,0,1,TRUE,0.5,0.8);
	set_domain_texture(mondo,bianco,"textures/sky2.hr",0,1,0,.5,FALSE,0.6,0.6);
	set_translate(strada, 0,0,0.01);
	set_x_rotate(mondo, 90,TRUE);

	char * tempp;
	scene = create_list(wily, "scene"); 
	scene = insert_in_list(scene, strada);
	scene = insert_in_list(scene, terra);
	scene = insert_in_list(scene, mondo);
	scene = insert_in_list(scene, cata);
	scene = insert_in_list(scene, cactus);
	scene = insert_in_list(scene, pozza);
	temp= create_copy(cactus, "cactus02");
	set_translate(temp, 33, 28, 0);
	scene = insert_in_list(scene, temp);
	temp= create_copy(cactus, "cactus03");
	set_translate(temp, -35, 30, 0);
	scene = insert_in_list(scene, temp);
	temp= create_copy(cactus, "cactus04");
	set_translate(temp, 30, -20, 0);
	scene = insert_in_list(scene, temp);
	temp= create_copy(cactus, "cactus05");
	set_translate(temp, -25, -30, 0);
	scene = insert_in_list(scene, temp);
	temp= create_copy(cactus, "cactus06");
	set_translate(temp, -10, 20, 0);
	scene = insert_in_list(scene, temp);
	temp= create_copy(cactus, "cactus07");
	set_translate(temp, 32, -58, 0);
	scene = insert_in_list(scene, temp);
	temp= create_copy(cactus, "cactus08");
	set_translate(temp, 0, 60, 0);
	scene = insert_in_list(scene, temp);
	temp= create_copy(cactus, "cactus09");
	set_translate(temp, 55, -80, 0); 
	scene = insert_in_list(scene, temp);
	temp= create_copy(cactus, "cactus10");
	set_translate(temp, 10, -35, 0);
	scene = insert_in_list(scene, temp);
	temp= create_copy(pozza, "pozza2");
	set_translate(temp, 35, -105, 0);
	scene = insert_in_list(scene, temp);

//********************** Lanciatore ***********************		
	temp = read_nurbs("lanciatore0.db");
	set_domain_texture(temp,legno,"textures/domainLauncher.hr",0,1,0,1,FALSE,.7, .7);
	set_z_rotate(temp, 90, TRUE);
	set_translate(temp, 0.,5.,1.);
	scene = insert_in_list(scene, temp);

//********************** LanciatoreB **********************	
	temp = read_nurbs("lanciatoreB0.db");
	set_attribute(temp,acciaio);
	set_z_rotate(temp, 90, TRUE);
	set_translate(temp, 0.,5.,1.);
	scene = insert_in_list(scene, temp);

//********************** Pedana ***************************
	temp=read_nurbs("pedana0.db");
	set_domain_texture(temp,acciaio,"textures/acciaio.hr",0,1,0,1,FALSE,0.7,0.7);
	set_z_rotate(temp, 90, TRUE);
	set_translate(temp, 0.,5.,1.);
	scene = insert_in_list(scene, temp);

//********************** Masso ****************************
	temp = read_nurbs("masso0.db");
	set_projection_texture(temp,rock,"textures/massoo.hr",0,TRUE,0.6,0.6);
// 		set_translate(temp, 10,-0.5,1);
 	set_z_rotate(temp, 90, TRUE);
 	set_translate(temp, 0.,5.,1.1); 
   	scene = insert_in_list(scene, temp);

	j=16; //indice di oscillazione del launcher
	k=46; //zoom
	l=0;   //indice traslazione masso


/* Inquadratura alternativa */
// 	sprintf(tempp,"willy.vw");  //setto il file vw
// 	file=fopen(tempp,"w");
// 	if(file){
// 		fprintf(file, "-33.640615 65.961822 49.694315\n");
// 		fprintf(file,"17.765000 4.767522 2.164820\n");
// 		fprintf(file,"0.201908 -0.489339 0.848399\n");
// 		fprintf(file,"%u\n", k);
// 		fprintf(file,"1\n");
// 		fclose(file);
// 	}

	file=fopen("willy.vw","w"); //imposto la camera
	if(file){
		fprintf(file, "-26.84402 63.08588 21.73079\n");
		fprintf(file,"17.765000 4.767522 2.164820\n");
		fprintf(file,"0.041 -0.288 0.956\n");
		fprintf(file,"%u\n", k);
		fprintf(file,"1\n");
		fclose(file);
	}
	save_scene("willy.md","scene_willy",scene);

   	for (i = 0; i<FRAMES; i++){
		if (i<=30){
			set_point_light(55, -55,55,1,1,1,4,70+i*3, sole);
		}
		 else {
			scene = create_list(wily, "scene"); 
			scene = insert_in_list(scene, strada);
			scene = insert_in_list(scene, terra);
			scene = insert_in_list(scene, mondo);
			scene = insert_in_list(scene, cata);
			scene = insert_in_list(scene, cactus);
			scene = insert_in_list(scene, pozza);
			temp= create_copy(cactus, "cactus02");
			set_translate(temp, 33, 28, 0);
			scene = insert_in_list(scene, temp);
			temp= create_copy(cactus, "cactus03");
			set_translate(temp, -35, 30, 0);
			scene = insert_in_list(scene, temp);
			temp= create_copy(cactus, "cactus04");
			set_translate(temp, 30, -20, 0);
			scene = insert_in_list(scene, temp);
			temp= create_copy(cactus, "cactus05");
			set_translate(temp, -25, -30, 0);
			scene = insert_in_list(scene, temp);
			temp= create_copy(cactus, "cactus06");
			set_translate(temp, -10, 20, 0);
			scene = insert_in_list(scene, temp);
			temp= create_copy(cactus, "cactus07");
			set_translate(temp, 32, -58, 0);
			scene = insert_in_list(scene, temp);
			temp= create_copy(cactus, "cactus08");
			set_translate(temp, 0, 60, 0);
			scene = insert_in_list(scene, temp);
			temp= create_copy(cactus, "cactus09");
			set_translate(temp, 55, -80, 0);
			scene = insert_in_list(scene, temp);
			temp= create_copy(cactus, "cactus10");
			set_translate(temp, 10, -35, 0);
			scene = insert_in_list(scene, temp);
			temp= create_copy(pozza, "pozza2");
			set_translate(temp, 35, -105, 0);
			scene = insert_in_list(scene, temp);

			if (i<47) {
		//********************** Lanciatore frames ***********************
				sprintf(tempp, "lanciatore%u.db", i-30);
				temp = read_nurbs(tempp);
				set_domain_texture(temp,legno,"textures/domainLauncher.hr",0,1,0,1,FALSE,.7, .7);
				set_z_rotate(temp, 90, TRUE);
				set_translate(temp, 0.,5.,1.);
				scene = insert_in_list(scene, temp);
		
		//********************** LanciatoreB frames **********************
				sprintf(tempp, "lanciatoreB%u.db", i-30);
				temp = read_nurbs(tempp);
				set_attribute(temp,acciaio);
				set_z_rotate(temp, 90, TRUE);
				set_translate(temp, 0.,5.,1.);
				scene = insert_in_list(scene, temp);
		
		//********************** Pedana frames ***************************
				sprintf(tempp, "pedana%u.db", i-30);
				temp = read_nurbs(tempp);
				set_domain_texture(temp,acciaio,"textures/acciaio.hr",0,1,0,1,FALSE,.7, .7);
				set_z_rotate(temp, 90, TRUE);
				set_translate(temp, 0.,5.,1.);
				scene = insert_in_list(scene, temp);

		//********************** Masso ****************************
				sprintf(tempp, "masso%u.db", i-30);
				temp = read_nurbs(tempp);
				set_projection_texture(temp,rock,"textures/massoo.hr",0,TRUE,0.6,0.6);
			// 		set_translate(temp, 10,-0.5,1);
				set_z_rotate(temp, 90, TRUE);
				set_translate(temp, 0.,5.,1.1); 
				scene = insert_in_list(scene, temp);
			}
			else {  //il launcher oscilla
				if (j == 12)
					segno = 1; 
				else if (j>=14) 
					segno = -1;
				j=j+segno;
	//********************** Lanciatore frames ***********************
				sprintf(tempp, "lanciatore%u.db", j);
				temp = read_nurbs(tempp);
				set_domain_texture(temp,legno,"textures/domainLauncher.hr",0,1,0,1,FALSE,.7, .7);
				set_z_rotate(temp, 90, TRUE);
				set_translate(temp, 0.,5.,1.);
				scene = insert_in_list(scene, temp);
	
	//********************** LanciatoreB frames **********************
				sprintf(tempp, "lanciatoreB%u.db", j);
				temp = read_nurbs(tempp);
				set_attribute(temp,acciaio);
				set_z_rotate(temp, 90, TRUE);
				set_translate(temp, 0.,5.,1.);
				scene = insert_in_list(scene, temp);
	
	//********************** Pedana frames ***************************
				sprintf(tempp, "pedana%u.db", j);
				temp = read_nurbs(tempp);
				set_domain_texture(temp,acciaio,"textures/acciaio.hr",0,1,0,1,FALSE,.7, .7);
				set_z_rotate(temp, 90, TRUE);
				set_translate(temp, 0.,5.,1.);
				scene = insert_in_list(scene, temp);
				if (i>80) {
					k=k-2;
					sprintf(tempp,"willy%u.vw", i);  //setto il file vw
					file=fopen(tempp,"w");
					if(file){
						fprintf(file, "-26.84402 63.08588 21.73079\n");
						fprintf(file,"17.765000 4.767522 2.164820\n");
						fprintf(file,"0.041 -0.288 0.956\n");
						fprintf(file,"%u\n", k);
						fprintf(file,"1\n");
						fclose(file);
					}
				}
			
			//********************** Masso ****************************
				l++;	
				temp = read_nurbs("masso15.db");
				set_projection_texture(temp,rock,"textures/massoo.hr",0,TRUE,0.6,0.6);
				set_z_rotate(temp, 90, TRUE);
				set_translate(temp, l,5.,1-cos(l*DEGTORAD)); 
				scene = insert_in_list(scene, temp);
			}
		} 
  		sprintf(tempp,"willy%u.md",i);
 		save_scene(tempp,"willy_anim",scene);
// 		save_scene_ani(FRAMES, "willy.ani", "willy", "scene", scene);
	}

	file=fopen("willy.ani","w");
	if (file){
		fprintf(file,"willy.ani\n");
		fprintf(file,"%u\n",FRAMES);
		for(i=0;i<FRAMES;i++)
			if (i<=80)
				fprintf(file,"%u willy%u.md willy.vw willy.arg\n",i ,i);
			else
				fprintf(file,"%u willy%u.md willy%u.vw willy.arg\n",i,i,i);	
		fclose(file);
	}

  	return(0);
}
